(function (Dashboard, $, Chartist, d3, moment, undefined) {
    'use strict';

    if ($.cookie('rtduration'))
        Dashboard.duration = parseInt($.cookie('rtduration')) || 60

    Dashboard.req_url = Dashboard.req_url || 'realtime.php'
    Dashboard.req_qs = Dashboard.req_qs || 'reqs'
    Dashboard.maxhist = Dashboard.maxhist || 600
    Dashboard.divisor = parseInt(Dashboard.divisor || 3)
    Dashboard.duration = parseInt(Dashboard.duration) || 60
    Dashboard.period = Dashboard.period || 1000
    Dashboard.rtchart_options = Dashboard.rtchart_options || {}

    function date_inter(value, i) {
        var step = Math.ceil(Dashboard.duration / Dashboard.divisor),
            offset = -Dashboard.duration + i * step
        if (i == Dashboard.divisor)
            return 'Now'
        else
            return '-' + moment(new Date(Date.now() + offset * 1000)).fromNow(true)
    }

    var stale, wait, xhr, rtchart,
        latest = 0,
        data = [[], [], [], []],
        rtchart_options = Object.assign(
          {}, {
              height: 255,
              width: 665,
              fullWidth: true,
              showArea: true,
              showLine: false,
              showPoint: false,
              stackBars: true,
              chartPadding: {top: 10, right: 15, bottom: 15, left: 15},
              plugins: [Chartist.plugins.legend({
                legendNames: [
                    'Served from Cache',
                    'Served from Backend',
                    'Synthetic Response',
                    'Blocked by WAF'
                ],
                clickable: false
              })],
              axisX: {
                type: Chartist.FixedScaleAxis,
                divisor: Dashboard.divisor,
                labelInterpolationFnc: date_inter,
                offset: 10,
                labelOffset: {x: -10, y: 0},
              },
              axisY: {
                type: Chartist.AutoScaleAxis,
                onlyInteger: true,
                referenceValue: 1,
                position: 'end',
                offset: 15,
                labelOffset: {x: 0, y: 4},
                scaleMinSpace: 40
              }
          }, Dashboard.rtchart_options)

    function update_donut(path, new_data, legend) {
        path.data(pie(new_data))
        path.transition().duration(750).attrTween("d", arc_tween)
        legend.selectAll('.legend_desc').text(
            function(d) {
                for (var i = 0; i < dataset1.length; i++) {
                    if (dataset1[i].label == d)
                        return dataset1[i].desc
                }
                return ''
            })
    }

    var seq = 0,
        delays = 10,
        durations = 250,
        delay = 250,
        seq_i = false,
        seq_n = false,
        seq_l = 0,
        no_animate = false

    function draw(d) {
        if (no_animate)
            return;

        seq++;

        if(d.type === 'bar') {
            if (!seq_n || seq_i != d.seriesIndex) {
                // console.log('set');
                seq_n = d.series.length
                seq_l = d.series.length
                seq_i = d.seriesIndex
            }

            seq_n--

            // console.log(d, seq_n, d.seriesIndex)

        // If the drawn element is a line we do a simple opacity fade in. This could also be achieved using CSS3 animations.
            d.element.animate({
              opacity: {
                // The delay when we like to start the animation
                begin: 0 + seq_n * 3, // 5 * seq_n, // * delays,
                // Duration of the (animation
                dur: 250, // 20 + seq_n * 20,
                // The value where the animation should start
                from: 0,
                // The value where it should end
                to: 0.8
              }
            });
        } else if(d.type === 'label' && d.axis === 'x') {
            d.element.animate({
              y: {
                begin: delay,  // seq * delays,
                dur: durations,
                from: d.y + 100,
                to: d.y,
                // We can specify an easing function from Chartist.Svg.Easing
                easing: 'easeOutQuart'
              }
            });
        } else if(d.type === 'label' && d.axis === 'y') {
            d.element.animate({
              x: {
                begin: delay,  // seq * delays,
                dur: durations,
                from: d.x - 100,
                to: d.x,
                easing: 'easeOutQuart'
              }
            });
        } else if(d.type === 'point') {
            d.element.animate({
              x1: {
                begin: delay,  // seq * delays,
                dur: durations,
                from: d.x - 10,
                to: d.x,
                easing: 'easeOutQuart'
              },
              x2: {
                begin: delay,  // seq * delays,
                dur: durations,
                from: d.x - 10,
                to: d.x,
                easing: 'easeOutQuart'
              },
              opacity: {
                begin: delay,  // seq * delays,
                dur: durations,
                from: 0,
                to: 1,
                easing: 'easeOutQuart'
              }
            });
        } else if(d.type === 'grid') {
            // Using d.axis we get x or y which we can use to construct our animation definition objects
            var pos1Animation = {
              begin: delay,  // seq * delays,
              dur: durations,
              from: d[d.axis.units.pos + '1'] - 30,
              to: d[d.axis.units.pos + '1'],
              easing: 'easeOutQuart'
            };

            var pos2Animation = {
              begin: delay,  // seq * delays,
              dur: durations,
              from: d[d.axis.units.pos + '2'] - 100,
              to: d[d.axis.units.pos + '2'],
              easing: 'easeOutQuart'
            };

            var animations = {};
            animations[d.axis.units.pos + '1'] = pos1Animation;
            animations[d.axis.units.pos + '2'] = pos2Animation;
            animations['opacity'] = {
              begin: delay,  // seq * delays,
              dur: durations,
              from: 0,
              to: 1,
              easing: 'easeOutQuart'
            };

            d.element.animate(animations);
        }
    }

    Dashboard.chart_donuts = function(d) {
        !d.handling || create_donut('#donut_handling', d.handling),
        !d.storage || create_donut('#donut_storage', d.storage),
        !d.bandwidth || create_donut('#donut_bandwidth', d.bandwidth)
    }

    var prior;

    Dashboard.chart_reqs = function(d, duration) {
        if (duration != undefined)
            Dashboard.duration = parseInt(duration) || 60

        if (!d || !d.series)
            return false

        var ts = parseInt(d.timestamp),
            fereq = d.series.fereq || [],
            bereq = d.series.bereq || [],
            synth = d.series.synth || [],
            wafbl = d.series.wafbl || []

        var start = 0

        for (var i in fereq) {
            if (fereq[i].x <= latest) {
                start++
            }
        }

        fereq = fereq.slice(start)
        bereq = bereq.slice(start)
        wafbl = wafbl.slice(start)
        synth = synth.slice(start)

        if (data[0].length > 0 && fereq.length > 0) {
            var n_fereq = fereq.slice(),
                next = fereq[0].x,
                last = data[0][data[0].length - 1].x

            if (next > last && last != (next - 1000)) {
                var mlen = (next - last) / 1000 - 1,
                    pre = Array.apply(null, Array(mlen)).map((v, i) => ({x: last + (i + 1) * 1000, y: 0}))

                fereq = pre.concat(fereq)
                bereq = pre.concat(bereq)
                wafbl = pre.concat(wafbl)
                synth = pre.map((v, i) => ({x: v.x, y: -1})).concat(synth)
                start = 0
            }
        }

        if (fereq.length > 0) {

            data[0].push.apply(data[0], fereq)
            data[0] = data[0].slice(-Dashboard.maxhist)

            data[1].push.apply(data[1], bereq)
            data[1] = data[1].slice(-Dashboard.maxhist)

            data[2].push.apply(data[2], synth)
            data[2] = data[2].slice(-Dashboard.maxhist)

            data[3].push.apply(data[3], wafbl)
            data[3] = data[3].slice(-Dashboard.maxhist)

            latest = fereq[fereq.length - 1].x
        }

        Dashboard.redraw()
    }

    Dashboard.data = function() {
        return [
            data[0].slice(-Dashboard.duration).map((v, i) => ({x: new Date(v.x), y: v.y})),
            data[1].slice(-Dashboard.duration).map((v, i) => ({x: new Date(v.x), y: v.y})),
            data[2].slice(-Dashboard.duration).map((v, i) => ({x: new Date(v.x), y: v.y})),
            data[3].slice(-Dashboard.duration).map((v, i) => ({x: new Date(v.x), y: v.y}))
        ]
    }

    Dashboard.put = function(d) {
        if (d.reqs)
            Dashboard.chart_reqs(d.reqs);

        if (d.donuts)
            Dashboard.chart_donuts(d.donuts);
    }

    Dashboard.stop = function() {
        !Dashboard.poll || clearInterval(Dashboard.poll)
    }

    Dashboard.do = function(cb, qs) {
        qs += '&after=' + latest
        if (!Dashboard.abort && (!xhr || xhr.readyState == 4))
            xhr = $.getJSON(Dashboard.req_url, qs || Dashboard.req_qs).done(cb)
    }

    Dashboard.start = function(period, qs, initqs) {
        Dashboard.period = period || Dashboard.period

        if (initqs) {
            Dashboard.do(Dashboard.put, initqs)
        }

        if (Dashboard.period > 0) {
            if (Dashboard.poll)
                clearInterval(Dashboard.poll)

            Dashboard.poll = setInterval(
                function() {Dashboard.do(Dashboard.put, qs)}, Dashboard.period)
        }
    }

    var created = false;

    Dashboard.redraw = function() {
        if (document.hidden)
            return false

        var stats = {'series': []}

        for (var i in data) {
            var d = data[i].slice(-Dashboard.duration)
            stats.series.push(d.map((v, i) => ({x: new Date(v.x), y: v.y})))
        }

        if (rtchart == undefined) {
            rtchart = new Chartist.Bar('.ct-chart', stats, rtchart_options)
            rtchart.duration = Dashboard.duration
            rtchart.on('created', function() { if (!created) { seq = 0; Dashboard.start(Dashboard.period, 'reqs&wait'); } })
            rtchart.on('draw', draw)
        } else {
            no_animate = true
            rtchart.update(stats, {}, true)
        }

        Dashboard.stale = false
    }

    Dashboard.do(Dashboard.put, 'reqs&donuts&n=' + Dashboard.maxhist)

    $(window).on('unload beforeunload', function() {
        Dashboard.abort = true
        // !Dashboard.poll || clearInterval(Dashboard.poll)
        // !Dashboard.wait || Dashboard.req.abort()
    })

    $(window).on('visibilitychange', function() {
        if (!document.hidden)
            Dashboard.redraw()
    })

    $('select[name=rtduration]').on('change', function() {
        Dashboard.duration = parseInt($(this).find(':selected').val()) || 60
        Dashboard.redraw()
    })
})(window.Dashboard = window.Dashboard || {}, jQuery, Chartist, d3, moment);


function create_donut(container, metadata, sub_metadata) {
    if (metadata == undefined)
        return

    var height = 265, width = 175;
    var radius = 60;
    var margin = 2;
    var thickness = 18;
    var offset_inner_y = 2;
    var offset_info_x = 1;
    var inner_title_y = 12;

    var legend_rect_size = 8,
        legend_rect_left_margin = 6,
        legend_rect_right_margin = 6

    var legend_title_height = 30,
        legend_title_top_margin = 10,
        legend_title_bottom_margin = 15

    var legend_row_height = 18;
    var legend_spacing = 2;
    var legend_btm_margin = 15;
    var donuts = [];
    var svg, svg_inner, legend, x_trans, y_trans,
        svg_main, values_main, colors_main, pie_main, arc_main, path_main, pie_main,
        svg_sub, values_sub, colors_sub, pie_sub, arc_sub, path_sub

    values_main = metadata.data.values
    colors_main = d3.scale.ordinal().range(metadata.data.colors)

    width = width + 2 * margin
    x_trans = width / 2
    y_trans = radius + margin

    if (sub_metadata != undefined) {
        values_sub = sub_metadata.data.values
        colors_sub = d3.scale.ordinal().range(sub_metadata.data.colors)

        height += (values_sub.length * legend_rect_size) + (values_sub.length - 1) * legend_spacing
        thickness = thickness / values_sub.length
    }

    svg = d3.select(container).append('svg')
                            .attr('width', width)
                            .attr('height', height)
                            .append('g')

    svg_main = svg.append('g')
                  .attr('transform', translate(width / 2, radius + margin))

    arc_main = d3.svg.arc()
                     .innerRadius(radius - thickness).outerRadius(radius)

    pie_main = d3.layout.pie()
                        .value(function(d) { return d.count; })
                        .sort(null)

    path_main = svg_main.selectAll('path')
                   .data(pie_main(values_main))
                   .enter().append('path')

    path_main.attr('fill', function(d, i) { return colors_main(d.data.label); })
             .attr("d", arc_main)
             .each(function(d) { this._arc = arc_main; this._current = d; })

    svg_inner = svg.append('g')
                   .attr('transform', translate(
                        (width / 2) + offset_info_x, (radius + margin) + offset_inner_y))

    svg_inner.append("text")
             .style('text-anchor', 'middle')
             .style('alignment-baseline', 'middle')
             .style('text-rendering', 'optimizeLegibility')
             .attr('y', inner_title_y)
             .text(metadata.inner_title)
             .attr('class', 'inner_title')

    var inner = svg_inner.append("text")
                         .style('text-anchor', 'middle')
                         .style('alignment-baseline', 'middle')
                         .style('text-rendering', 'optimizeLegibility')
                         .attr('class', 'inner_title')

    inner.append("tspan")
         .attr('class', 'inner_value')
         .text(metadata.inner_value)

    inner.append("tspan")
         .attr('class', 'inner_units')
         .attr('dx', 3)
         .text(metadata.inner_units)

    // legend title
    svg_main.append("text")
            .attr('y', radius + legend_title_height + margin)
            .attr('height', legend_title_height)
            .attr('class', 'legend_title')
            .style('text-anchor', 'middle')
            .text(metadata.title)

    // legend rows
    legend = svg_main.selectAll('.legend')
                     .data(function() { return colors_main.domain() })
                     .enter()
                     .append('g')
                     .attr('class', 'legend')
                     .attr('transform', function(d, i) {
                         return translate(
                            - width / 2,
                            radius + legend_title_height + legend_title_top_margin + legend_title_bottom_margin + i * legend_row_height
                         )})

    // legend row circles
    legend.append('rect')
              .attr('width', legend_rect_size)
              .attr('height', legend_rect_size)
              .attr('y', - legend_rect_size)
              .attr('x', legend_rect_left_margin)
              .attr('rx', 5)
              .attr('ry', 5)
              .style('fill', colors_main)
              .style('stroke', colors_main)

    // legend row labels
    legend.append('text')
          .attr('x', legend_rect_size + legend_rect_right_margin + legend_rect_left_margin)
          .text(function(d) { return d; })

    // legend row values
    legend.append('text')
          .attr('x', (width - 2 * margin - 10))
          .attr('class', 'legend_desc')
          .style('text-anchor', 'end')
          .style('fill', '#989898')
          .text(function(d) {
                    for (var i = 0; i < values_main.length; i++)
                        if (values_main[i].label == d)
                            return values_main[i].desc }
                )

    donuts.push({'pie': pie_main, 'path': path_main, 'legend': legend})
    return donuts
}

function translate(x, y) {
    return 'translate(' + x + ',' + y + ')'
}

function arc_tween(a) {
    var i = d3.interpolate(this._current, a)
    this._current = i(0)
    var arc = this._arc
    return function(t) {
        return arc(i(t));
    }
}
